unit RyuMPEG;

interface

uses
  Classes, SysUtils, Graphics;

const
  AVCODEC_MAX_AUDIO_FRAME_SIZE = 192000; // 1 second of 48khz 32bit audio

  PIXEL_FORMAT = pf32bit;
  PIXEL_SIZE = 4;

  UNKNOWN_PACKET = 0;
  VIDEO_PACKET = 1;
  AUDIO_PACKET = 2;

type
  TVideoInfo = packed record
	  CodecID, PixelFormat, Width, Height, fps, bitrate, frameSize : integer;
  end;

  TAudioInfo = packed record
	  CodecID, SampleRate, Channels : integer;
  end;

  TMediaInfo = packed record
    VideoInfoString : packed array [0..1024*16-1] of AnsiChar;
    AudioInfoString : packed array [0..1024*16-1] of AnsiChar;

    Duration : int64;

    VideoInfo : TVideoInfo;
    AudioInfo : TAudioInfo;
  end;
  PMediaInfo = ^TMediaInfo;

function  openRyuMPEG(AFileName:PAnsiChar; var AErrorCode:integer):pointer;
          cdecl; external 'libRyuMPEG.dll';

procedure closeRyuMPEG(AHandle:pointer);
          cdecl; external 'libRyuMPEG.dll';

procedure getVideoInfo(AHandle:pointer; var AVideoInfo:TVideoInfo);
          cdecl; external 'libRyuMPEG.dll';

procedure getAudioInfo(AHandle:pointer; var AAudioInfo:TAudioInfo);
          cdecl; external 'libRyuMPEG.dll';

procedure getMediaInfo(AHandle:pointer; var AMediaInfo:TMediaInfo);
          cdecl; external 'libRyuMPEG.dll';

function  readData(AHandle:pointer; AData:pointer; var ASize:integer;
          var APacketType,APostion:integer):boolean;
          cdecl; external 'libRyuMPEG.dll';

function  decodeVideoData(AHandle:pointer; AData:pointer; ASize:integer;
          ABitmap:pointer):boolean;
          cdecl; external 'libRyuMPEG.dll';

procedure BitmapUpsideDown(AData:pointer; ABitmap:TBitmap);

implementation

procedure BitmapUpsideDown(AData:pointer; ABitmap:TBitmap);
var
  pSrc, pDst : PByte;
  Loop, iLineSize : Integer;
begin
  Assert(ABitmap.PixelFormat = PIXEL_FORMAT, 'ABitmap.PixelFormat <> PIXEL_FORMAT');

  pSrc := AData;
  pDst := ABitmap.ScanLine[0];

  iLineSize := ABitmap.Width * PIXEL_SIZE;

  for Loop := 0 to ABitmap.Height - 1 do begin
    Move(pSrc^, pDst^, iLineSize);

    Inc(pSrc, iLineSize);
    Dec(pDst, iLineSize);
  end;
end;

end.

